@@ -22,8 +22,8 @@
  * them of the latest execution. Entry eviction is done in the same
  * way to pg_stat_statements.
  *
- * Copyright (c) 2008-2020, PostgreSQL Global Development Group
- * Copyright (c) 2012-2021, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
+ * Copyright (c) 2008-2022, PostgreSQL Global Development Group
+ * Copyright (c) 2012-2022, NIPPON TELEGRAPH AND TELEPHONE CORPORATION
  *
  * IDENTIFICATION
  *	  pg_store_plans/pg_store_plans.c
@@ -71,7 +71,7 @@ PG_MODULE_MAGIC;
 static const uint32 PGSP_PG_MAJOR_VERSION = PG_VERSION_NUM / 100;
 
 /* This constant defines the magic number in the stats file header */
-static const uint32 PGSP_FILE_HEADER = 0x20211125;
+static const uint32 PGSP_FILE_HEADER = 0x20221214;
 static int max_plan_len = 5000;
 
 /* XXX: Should USAGE_EXEC reflect execution time and/or buffer usage? */
@@ -98,7 +98,8 @@ typedef uint32 queryid_t;
 typedef enum pgspVersion
 {
 	PGSP_V1_5 = 0,
-	PGSP_V1_6
+	PGSP_V1_6,
+	PGSP_V1_7
 } pgspVersion;
 
 /*
@@ -141,6 +142,10 @@ typedef struct Counters
 	int64		temp_blks_written;	/* # of temp blocks written */
 	double		blk_read_time;		/* time spent reading, in msec */
 	double		blk_write_time; 	/* time spent writing, in msec */
+	double		temp_blk_read_time;	/* time spent reading temp blocks,
+									   in msec */
+	double		temp_blk_write_time;/* time spent writing temp blocks,
+									   in msec */
 	TimestampTz	first_call;			/* timestamp of first call  */
 	TimestampTz	last_call;			/* timestamp of last call  */
 	double		usage;				/* usage factor */
@@ -313,6 +318,7 @@ PG_FUNCTION_INFO_V1(pg_store_plans_reset);
 PG_FUNCTION_INFO_V1(pg_store_plans_hash_query);
 PG_FUNCTION_INFO_V1(pg_store_plans);
 PG_FUNCTION_INFO_V1(pg_store_plans_1_6);
+PG_FUNCTION_INFO_V1(pg_store_plans_1_7);
 PG_FUNCTION_INFO_V1(pg_store_plans_shorten);
 PG_FUNCTION_INFO_V1(pg_store_plans_normalize);
 PG_FUNCTION_INFO_V1(pg_store_plans_jsonplan);
@@ -331,6 +337,11 @@ PG_FUNCTION_INFO_V1(pg_store_plans_info);
 #define ROLE_PG_READ_ALL_STATS		DEFAULT_ROLE_READ_ALL_STATS
 #endif
 
+#if PG_VERSION_NUM >= 150000
+static shmem_request_hook_type prev_shmem_request_hook = NULL;
+#endif
+
+static void pgsp_shmem_request(void);
 static void pgsp_shmem_startup(void);
 static void pgsp_shmem_shutdown(int code, Datum arg);
 static void pgsp_ExecutorStart(QueryDesc *queryDesc, int eflags);
@@ -381,7 +392,6 @@ _PG_init(void)
 	 */
 	if (!process_shared_preload_libraries_in_progress)
 		return;
-
 #if PG_VERSION_NUM >= 140000
 	/*
 	 * Inform the postmaster that we want to enable query_id calculation if
@@ -536,17 +546,17 @@ _PG_init(void)
 
 	EmitWarningsOnPlaceholders("pg_store_plans");
 
-	/*
-	 * Request additional shared resources.  (These are no-ops if we're not in
-	 * the postmaster process.)  We'll allocate or attach to the shared
-	 * resources in pgsp_shmem_startup().
-	 */
-	RequestAddinShmemSpace(shared_mem_size());
-	RequestNamedLWLockTranche("pg_store_plans", 1);
+#if PG_VERSION_NUM < 150000	
+	pgsp_shmem_request();
+#endif
 
 	/*
 	 * Install hooks.
 	 */
+#if PG_VERSION_NUM >= 150000
+	prev_shmem_request_hook = shmem_request_hook;
+	shmem_request_hook = pgsp_shmem_request;
+#endif
 	prev_shmem_startup_hook = shmem_startup_hook;
 	shmem_startup_hook = pgsp_shmem_startup;
 	prev_ExecutorStart = ExecutorStart_hook;
@@ -576,6 +586,22 @@ _PG_fini(void)
 	ProcessUtility_hook = prev_ProcessUtility;
 }
 
+/*
+ * pgsp_shmem_request: request shared memory to the core.
+ * Called as a hook in PG15 or later, otherwise called from _PG_init().
+ */
+static void
+pgsp_shmem_request(void)
+{
+#if PG_VERSION_NUM >= 150000
+	if (prev_shmem_request_hook)
+		prev_shmem_request_hook();
+#endif
+
+	RequestAddinShmemSpace(shared_mem_size());
+	RequestNamedLWLockTranche("pg_store_plans", 1);
+}
+
 /*
  * shmem_startup hook: allocate or attach to shared memory,
  * then load any pre-existing statistics from file.
@@ -1327,8 +1353,12 @@ pgsp_store(char *plan, queryid_t queryId,
 	e->counters.local_blks_written += bufusage->local_blks_written;
 	e->counters.temp_blks_read += bufusage->temp_blks_read;
 	e->counters.temp_blks_written += bufusage->temp_blks_written;
+
 	e->counters.blk_read_time += INSTR_TIME_GET_MILLISEC(bufusage->blk_read_time);
 	e->counters.blk_write_time += INSTR_TIME_GET_MILLISEC(bufusage->blk_write_time);
+	e->counters.temp_blk_read_time += INSTR_TIME_GET_MILLISEC(bufusage->temp_blk_read_time);
+	e->counters.temp_blk_write_time += INSTR_TIME_GET_MILLISEC(bufusage->temp_blk_write_time);
+
 	e->counters.last_call = GetCurrentTimestamp();
 	e->counters.usage += USAGE_EXEC(total_time);
 
@@ -1359,11 +1389,20 @@ pg_store_plans_reset(PG_FUNCTION_ARGS)
 /* Number of output arguments (columns) for various API versions */
 #define PG_STORE_PLANS_COLS_V1_5	27
 #define PG_STORE_PLANS_COLS_V1_6	26
-#define PG_STORE_PLANS_COLS			27	/* maximum of above */
+#define PG_STORE_PLANS_COLS_V1_7	28
+#define PG_STORE_PLANS_COLS			28	/* maximum of above */
 
 /*
  * Retrieve statement statistics.
  */
+Datum
+pg_store_plans_1_7(PG_FUNCTION_ARGS)
+{
+	pg_store_plans_internal(fcinfo, PGSP_V1_7);
+
+	return (Datum) 0;
+}
+
 Datum
 pg_store_plans_1_6(PG_FUNCTION_ARGS)
 {
@@ -1616,11 +1655,19 @@ pg_store_plans_internal(FunctionCallInfo fcinfo,
 		values[i++] = Int64GetDatumFast(tmp.temp_blks_written);
 		values[i++] = Float8GetDatumFast(tmp.blk_read_time);
 		values[i++] = Float8GetDatumFast(tmp.blk_write_time);
+
+		if (api_version >= PGSP_V1_7)
+		{
+			values[i++] = Float8GetDatumFast(tmp.temp_blk_read_time);
+			values[i++] = Float8GetDatumFast(tmp.temp_blk_write_time);
+		}
+
 		values[i++] = TimestampTzGetDatum(tmp.first_call);
 		values[i++] = TimestampTzGetDatum(tmp.last_call);
 
 		Assert(i == (api_version == PGSP_V1_5 ? PG_STORE_PLANS_COLS_V1_5 :
 					 api_version == PGSP_V1_6 ? PG_STORE_PLANS_COLS_V1_6 :
+					 api_version == PGSP_V1_7 ? PG_STORE_PLANS_COLS_V1_7 :
 					 -1 /* fail if you forget to update this assert */ ));
 
 		tuplestore_putvalues(tupstore, tupdesc, values, nulls);
